#ifndef PREPROCESS_H
#define PREPROCESS_H

#include "petscmat.h"

#define TRUTH(x) ( (x) ? PETSC_TRUE : PETSC_FALSE )

//struct NumericalSolution {
//  void* dummy;
//} ;

typedef struct SystemPreprocessor_* SystemPreprocessor;
typedef struct NumericalProblem_* NumericalProblem;
typedef struct NumericalSolution_* NumericalSolution;
typedef PetscReal SuitabilityValue;

typedef struct SalsaTransform_* SalsaTransform;
typedef struct SalsaTransformObject_* SalsaTransformObject;

extern PetscErrorCode SysProInitialize();
extern PetscErrorCode SysProFinalize();
extern PetscErrorCode DeclarePreprocessor
(char *name,
 PetscErrorCode(*global_setup)(),
 PetscErrorCode(*specific_setup)(NumericalProblem,SalsaTransform),
 PetscErrorCode(*specific_unset)(NumericalProblem),
 PetscErrorCode(*global_unset)(),
 PetscErrorCode(*ctxcreate)(NumericalProblem,void**),
 PetscErrorCode(*ctxdelete)(void*),
 PetscErrorCode(*start_function)(char*,int,PetscTruth,NumericalProblem,NumericalProblem*,void*,void**,PetscTruth*),
 PetscErrorCode(*end_function)
 (char*,PetscTruth,void*,void*,NumericalProblem,NumericalProblem,NumericalSolution,NumericalSolution)
 );
extern PetscErrorCode DeclarePreprocessorIntelligentChoice
(char *name,PetscErrorCode(*picker)(NumericalProblem,char**,char**));
extern PetscErrorCode PreprocessorsOptionsHandling();

extern PetscErrorCode SysProDeclareFunctions
(PetscErrorCode(*classstaticsetup)(char*),
 PetscErrorCode(*classdynamicsetup)(char*,NumericalProblem),
 PetscErrorCode(*classproblemcloner)(char*,char*,int,NumericalProblem,NumericalProblem),
 PetscErrorCode(*solver)(NumericalProblem,void*,NumericalSolution*),
 PetscErrorCode(*problemdelete)(NumericalProblem),
 PetscErrorCode(*solutioncreator)(NumericalProblem,NumericalSolution*),
 PetscErrorCode(*solutioncopy)(NumericalSolution,NumericalSolution),
 PetscErrorCode(*solutiondelete)(NumericalSolution),
 PetscErrorCode(*ctxcloner)(char*,char*,void*,void**),
 PetscErrorCode(*ctxfree)(void*),
 PetscErrorCode(*solutioncontextdelete)(NumericalSolution)
 );

extern PetscErrorCode ProcessPreprocessorOptions(char *processor,void *ctx);

extern PetscErrorCode PreprocessorGetIndex(char*,int*);
extern PetscErrorCode SystemPreprocessorGetByName(char*,SystemPreprocessor*);
extern PetscErrorCode PreprocessorGetSetting(char*,char**,int*);

extern PetscErrorCode RetrievePreprocessorChoice(int,char**,int*);
extern PetscErrorCode GetFirstPreprocessor(char **preprocess);
extern PetscErrorCode GetNextPreprocessor(char **next_one);

extern PetscErrorCode SuccessorPreprocessor(char *,char**);
extern PetscErrorCode InitRetrievingPreprocessors();
extern PetscErrorCode StartRetrievingCurrentPreprocessors(char**,char**,int*,PetscTruth*);
extern PetscErrorCode ContinueRetrievingCurrentPreprocessors(char**,char**,int*,PetscTruth*);
extern PetscErrorCode StartRetrievingAllPreprocessors(char**,char***,int*,PetscTruth*);
extern PetscErrorCode ContinueRetrievingAllPreprocessors(char**,char***,int*,PetscTruth*);
extern PetscErrorCode RetrieveAllPreprocessorValues(char*,char***,int*);

extern PetscErrorCode RegisterPreprocessorContext(char *pre,void *ctx);
extern PetscErrorCode PreprocessorGetContext(char *pre,void **ctx);

extern PetscErrorCode PreprocessedSolution
(char*,NumericalProblem,void*,NumericalSolution*);
extern PetscErrorCode PreprocessedProblemSolving
(NumericalProblem,NumericalSolution*);

/*
 * Anamod interface
 */
extern PetscErrorCode PreprocessorSetPreservedCategories(char*,char*);
extern PetscErrorCode PreprocessorGetPreservedCategories(char*,char**);
extern PetscErrorCode SysProComputeQuantity
(NumericalProblem,char*,char*,void*,int*,PetscTruth*);
extern PetscErrorCode SysProRetrieveQuantity
(NumericalProblem,char*,char*,void*,int*,PetscTruth*);
extern PetscErrorCode SysProRemoveQuantity
(NumericalProblem,char*,char*,PetscTruth*);
extern PetscErrorCode SysProFreeQuantities(NumericalProblem);

/*
 * Tracing
 */
extern PetscErrorCode SysProDefaultTrace(void *ctx,char *fmt,va_list argp);
extern PetscErrorCode SysProDeclareTraceFunction
(PetscErrorCode(*fn)(void*,char*,va_list));
extern PetscErrorCode SysProDeclareTraceContext(void *ctx);
extern PetscErrorCode SysProTraceMessage(char *fmt,...);
extern PetscErrorCode SysProHasTrace(PetscTruth *flg);
extern PetscErrorCode SysProDeclareProblemMonitor
(PetscErrorCode(*)(NumericalProblem));
extern PetscErrorCode SysProDeclareErrorTracer
(PetscErrorCode(*)(NumericalProblem,NumericalSolution,char*));
extern PetscErrorCode SysProGetErrorTracer
(PetscErrorCode(**)(NumericalProblem,NumericalSolution,char*));

/*
 * Output
 */
extern PetscErrorCode TabReportAllPreprocessors(char **key,int);
extern PetscErrorCode TabReportActivePreprocessors(char **key,char **val,int);
extern PetscErrorCode ReportEnabledPreprocessors(char *name);
extern PetscErrorCode ReportSysProCallStackState(char *name);
extern PetscErrorCode ScreenOutputTab(char *key,char *val);
extern PetscErrorCode ScreenOutputTabLine(char *key,char *val);

/* Utility stuff */
extern PetscErrorCode NumericalProblemGetComm(NumericalProblem,MPI_Comm*);

#endif
