#ifndef PREPROCESS_IMPL_H
#define PREPROCESS_IMPL_H

#include "petsc.h"
#include "syspro.h"

#define SYSPROCHECKVALID(i,c,s) {if (!i) SETERRQ1(1,"Null pointer for <%s>",s); if (i->cookie!=c) SETERRQ1(1,"Not a valid <%s>",s);}
#define SYSPROCHECKVALIDa(i,c,s,a) {if (!i) SETERRQ2(1,"Null pointer for <%s>, argument %d",s,a); if (i->cookie!=c) SETERRQ2(1,"Not a valid <%s>, argument %d",s,a);}

#define NUMERICALPROBLEMHEADER \
  MPI_Comm comm; void *ctx;

struct NumericalProblem_ {
  NUMERICALPROBLEMHEADER
};

struct SystemPreprocessor_ {
  char *name;
  SalsaTransform transform;
  char *preserved,*required;
  PetscTruth exhaustive;
  PetscErrorCode(*setup)(NumericalProblem,SalsaTransform);
  PetscErrorCode(*unset)(NumericalProblem);
  PetscErrorCode(*ctxcreate)(NumericalProblem,void**);
  PetscErrorCode(*ctxdelete)(void*);
  PetscErrorCode(*start_function)(char*,int,PetscTruth,NumericalProblem,NumericalProblem*,void*,void**,PetscTruth*);
  PetscErrorCode(*end_function)(char*,PetscTruth,void*,void*,NumericalProblem,NumericalProblem,NumericalSolution,NumericalSolution);
  PetscErrorCode(*optionshandling)();
  PetscErrorCode(*intelligence)(NumericalProblem,char**,char**);
};

struct SalsaTransformObject_ {
  char *name,*explanation;
  SalsaTransform transform;
  int n_options,alloc_options,*options; char *option,**optionexplanation;
  int *options_marked; int active_option;
  int alloc_annotate_c; char **annotate_c;
  int alloc_annotate_i,*annotate_i;
  int marked;
  PetscErrorCode(*suitabilityfunction)
    (NumericalProblem,void*,SuitabilityValue*);
  void *suitabilityctx;
};

struct SalsaTransform_ {
  char *name; PetscTruth userchoices;
  SalsaTransformObject *transformobjects;
  int alloc_objects,n_objects,*aprioriselection;
  int n_annotate_c; char **annotations_c;
  int n_annotate_i; char **annotations_i;
} ;

#endif
