#ifndef LINEAR_H
#define LINEAR_H

#include "petscmat.h"
#include "petscksp.h"
#include "syspro.h"
#include "nmd.h"

typedef struct LinearSystem_* LinearSystem;
typedef struct LinearSolution_* LinearSolution;
typedef struct Diagnostics_* Diagnostics;

/*
 * Linear system object
 */
extern PetscErrorCode CreateLinearSystem(MPI_Comm,LinearSystem*);
extern PetscErrorCode DeleteLinearSystem(LinearSystem);
extern PetscErrorCode LinearSystemSetParts(LinearSystem,Mat,Mat,Vec,Vec,Vec);
extern PetscErrorCode LinearSystemInheritParts(LinearSystem,Mat,Mat,Vec,Vec,Vec);
extern PetscErrorCode LinearSystemGetParts(LinearSystem,Mat*,Mat*,Vec*,Vec*,Vec*);
extern PetscErrorCode LinearSystemGetTmpVector(LinearSystem,Vec*);
extern PetscErrorCode LinearSystemSetContext(LinearSystem,void *ctx);
extern PetscErrorCode LinearSystemGetContext(LinearSystem,void **ctx);
extern PetscErrorCode LinearSystemSetKnownSolution(LinearSystem,PetscTruth);
extern PetscErrorCode LinearSystemGetKnownSolution(LinearSystem,PetscTruth*);
extern PetscErrorCode LinearSystemSetMetadata(LinearSystem,NMD_metadata);
extern PetscErrorCode LinearSystemGetMetadata(LinearSystem,NMD_metadata*);

extern PetscErrorCode LinearSystemDuplicatePointers(LinearSystem,LinearSystem*);
extern PetscErrorCode LinearSystemDuplicate(LinearSystem,LinearSystem*);
extern PetscErrorCode LinearSystemCopy(LinearSystem,LinearSystem);

/*
 * Linear solution object
 */
extern PetscErrorCode CreateLinearSolution(LinearSolution*);
extern PetscErrorCode LinearCreateNumericalSolution(NumericalProblem,NumericalSolution*);
extern PetscErrorCode LinearSolutionDelete(LinearSolution);
extern PetscErrorCode LinearSolutionCopy(LinearSolution,LinearSolution);
extern PetscErrorCode LinearSolutionCopyStats(LinearSolution,LinearSolution);
extern PetscErrorCode LinearCopyNumericalSolution(NumericalSolution,NumericalSolution);
extern PetscErrorCode LinearDeleteNumericalSolution(NumericalSolution);
extern PetscErrorCode LinearDeleteNumericalSolutionContext(NumericalSolution);
extern PetscErrorCode CreateDefaultLinearSolution(NumericalProblem,NumericalSolution*);
extern PetscErrorCode LinearSolutionSetVector(LinearSolution,Vec);
extern PetscErrorCode LinearSolutionGetVector(LinearSolution,Vec*);
extern PetscErrorCode LinearSolutionGetStatistics(LinearSolution,NMD_metadata*);
extern PetscErrorCode LinearSolutionSetTimes(LinearSolution,PetscLogDouble,PetscLogDouble,PetscLogDouble);
extern PetscErrorCode LinearSolutionAddToPreprocessTime(LinearSolution,PetscLogDouble);
extern PetscErrorCode LinearSolutionGetTimes(LinearSolution,PetscLogDouble*,PetscLogDouble*,PetscLogDouble*);
extern PetscErrorCode LinearSolutionSetContext(LinearSolution,void *ctx);
extern PetscErrorCode LinearSolutionGetContext(LinearSolution,void **ctx);
extern PetscErrorCode LinearSystemTrueDistance
(LinearSystem,LinearSolution,PetscReal*);
extern PetscErrorCode LinearSystemTrueDistancePrint
(NumericalProblem,NumericalSolution,char*);
extern PetscErrorCode LinearSolutionCreateStatistics(LinearSolution sol);

extern PetscErrorCode PreprocessedLinearSystemSolution(LinearSystem,LinearSolution*);

extern PetscErrorCode delete_diagnostics(Diagnostics);
extern PetscErrorCode make_diagnostics(char*,char*,Diagnostics*);

/*
 * Declaration of linear solver preprocessors
 */
extern PetscErrorCode DeclareSingletonPreprocessor(void);
extern PetscErrorCode DeclareFlipsignPreprocessor(void);
extern PetscErrorCode DeclareApproximationPreprocessor(void);
extern PetscErrorCode DeclareDummyRowPreprocessor(void);
extern PetscErrorCode DeclareDistributionPreprocessor(void);
extern PetscErrorCode DeclareScalingPreprocessor(void);
extern PetscErrorCode DeclarePCPreprocessor(void);
extern PetscErrorCode DeclareKSPPreprocessor(void);

#endif
