#ifndef TRANSFORM_H
#define TRANSFORM_H

#include "syspro.h"
#include "petsc.h"

extern PetscErrorCode NewTransform(char *name,SalsaTransform*);
extern PetscErrorCode DeregisterTransform(SalsaTransform);
extern PetscErrorCode TransformGetName(SalsaTransform,char**);
extern PetscErrorCode TransformGetByName(char*,SalsaTransform*);

extern PetscErrorCode NewTransformObject(char*,char*,SalsaTransformObject*);
extern PetscErrorCode FreeTransformObject(SalsaTransformObject tf);
extern PetscErrorCode TransformObjectGetByName(char*,char*,SalsaTransformObject*);
extern PetscErrorCode TransformGetObjects(SalsaTransform,int*,SalsaTransformObject**);
extern PetscErrorCode TransformObjectGetName(SalsaTransformObject,char**);
extern PetscErrorCode TransformObjectGetTransformName(SalsaTransformObject,char**);
extern PetscErrorCode TransformObjectsGetNames(SalsaTransform,char***);
extern PetscErrorCode TransformObjectSetExplanation(SalsaTransformObject,char*);
extern PetscErrorCode TransformObjectSetSuitabilityFunction
(SalsaTransformObject,void*,PetscErrorCode(*)(NumericalProblem,void*,SuitabilityValue*));
extern PetscErrorCode TransformObjectGetSuitabilityFunction
(SalsaTransformObject,void **,PetscErrorCode(**)(NumericalProblem,void*,SuitabilityValue*));
extern PetscErrorCode TransformObjectAddOptionExplanation(SalsaTransformObject,int,char*);

extern PetscErrorCode TransformItemGetFirstOption(char*,char*,int*,PetscTruth*);
extern PetscErrorCode TransformItemGetNextOption(char*,char*,int*,PetscTruth*);


extern PetscErrorCode TransformReportTeXTable(SalsaTransform,FILE*);
extern PetscErrorCode TransformReportEnabled(SalsaTransform,char**);
extern PetscErrorCode TransformItemDescribeShort(SalsaTransform,char*,int,char**);
extern PetscErrorCode TransformItemDescribeLong(SalsaTransform,char*,int,char**);

/*
 * Marking
 */
extern PetscErrorCode TransformObjectMark(SalsaTransformObject tf);
extern PetscErrorCode TransformObjectUnmark(SalsaTransformObject tf);
extern PetscErrorCode TransformObjectsMarkAll(SalsaTransform tf);
extern PetscErrorCode TransformObjectsUnmarkAll(SalsaTransform tf);
extern PetscErrorCode TransformObjectGetMark(SalsaTransformObject,int*);
extern PetscErrorCode TransformItemOptionMark(SalsaTransform,char*,int);
extern PetscErrorCode TransformGetNUnmarked(SalsaTransform,int*);
extern PetscErrorCode TransformObjectsUseOnly(SalsaTransform,char *list);
extern PetscErrorCode TransformItemOptionsUseOnly(SalsaTransformObject,char*);
extern PetscErrorCode TransformGetNItems(SalsaTransform,int *n);
extern PetscErrorCode TransformGetNextUnmarkedItem
(SalsaTransform,char*,SalsaTransformObject*,PetscTruth*);
extern PetscErrorCode PreprocessorSaveAprioriSelection(SystemPreprocessor);
extern PetscErrorCode PreprocessorApplyAprioriSelection(SystemPreprocessor);

/*
 * annotations
 */
extern PetscErrorCode SysProDefineCharAnnotation(char*,char*);
extern PetscErrorCode TransformCharAnnotationGetIndex(SalsaTransform,char*,int*,PetscTruth*);
extern PetscErrorCode TransformObjectCharAnnotate(SalsaTransformObject,char*,char*);
extern PetscErrorCode TransformObjectIntAnnotate(SalsaTransformObject tf,char*,int);
extern PetscErrorCode TransformObjectGetIntAnnotation
(SalsaTransformObject,char *an,int *v,PetscTruth *f);
extern PetscErrorCode SysProDefineIntAnnotation(char*,char*);
extern PetscErrorCode TransformIntAnnotationGetIndex(SalsaTransform,char*,int*,PetscTruth*);
extern PetscErrorCode TransformItemIntAnnotate(SalsaTransform,int,int);
extern PetscErrorCode TransformItemGetIntAnnotation
(SalsaTransform,int idx,char *an,int *v,PetscTruth *f);

/*
 * options
 */
extern PetscErrorCode TransformObjectDefineOption(SalsaTransformObject,char*);
extern PetscErrorCode TransformObjectAddOption(SalsaTransformObject,int);

extern PetscErrorCode TransformItemCharAnnotationGetIndex(SalsaTransform,char*,int*);
extern PetscErrorCode TransformItemGetCharAnnotation
(SalsaTransform,int idx,char *an,char **v,PetscTruth*);


extern PetscErrorCode TransformCurrentItemDefineOption(SalsaTransform,char*,char*);
extern PetscErrorCode TransformItemDefineOption(SalsaTransform,int,char*,char*);
extern PetscErrorCode TransformItemGetNOptions(SalsaTransform,int it,int *nopt);
extern PetscErrorCode TransformItemGetOptionI(SalsaTransform,int it,int iopt,int *v);

extern PetscErrorCode TransformSetUserChoices(SalsaTransform,PetscTruth);
extern PetscErrorCode TransformGetUserChoices(SalsaTransform,PetscTruth*);

#endif
